@@ -10,7 +10,8 @@ from trytond.pool import Pool, PoolMeta
 
 from trytond.modules.product import price_digits
 
-__all__ = ['Configuration', 'ConfigurationSaleMethod', 'Sale', 'SaleLine']
+__all__ = ['Configuration', 'ConfigurationSaleMethod', 'Sale', 'SaleLine',
+    'ReturnSale', 'Promotion']
 sale_shipment_cost_method = fields.Selection(
         'get_sale_shipment_cost_methods', "Sale Shipment Cost Method")
 
@@ -24,8 +25,7 @@ def get_sale_methods(field_name):
     return func
 
 
-class Configuration:
-    __metaclass__ = PoolMeta
+class Configuration(metaclass=PoolMeta):
     __name__ = 'sale.configuration'
     sale_shipment_cost_method = fields.MultiValue(sale_shipment_cost_method)
     get_sale_shipment_cost_methods = get_sale_methods('shipment_cost_method')
@@ -43,8 +43,7 @@ class Configuration:
             'sale_shipment_cost_method').default_sale_shipment_cost_method()
 
 
-class ConfigurationSaleMethod:
-    __metaclass__ = PoolMeta
+class ConfigurationSaleMethod(metaclass=PoolMeta):
     __name__ = 'sale.configuration.sale_method'
     sale_shipment_cost_method = sale_shipment_cost_method
     get_sale_shipment_cost_methods = get_sale_methods('shipment_cost_method')
@@ -54,7 +53,7 @@ class ConfigurationSaleMethod:
         TableHandler = backend.get('TableHandler')
         exist = TableHandler.table_exist(cls._table)
         if exist:
-            table = TableHandler(cls, module_name)
+            table = cls.__table_handler__(module_name)
             exist &= table.column_exist('sale_shipment_cost_method')
 
         super(ConfigurationSaleMethod, cls).__register__(module_name)
@@ -74,8 +73,7 @@ class ConfigurationSaleMethod:
         return 'order'
 
 
-class Sale:
-    __metaclass__ = PoolMeta
+class Sale(metaclass=PoolMeta):
     __name__ = 'sale.sale'
     carrier = fields.Many2One('carrier', 'Carrier',
         domain=[
@@ -119,8 +117,12 @@ class Sale:
                 and self.warehouse.address
                 and self.warehouse.address.country):
             pattern['from_country'] = self.warehouse.address.country.id
+        else:
+            pattern['from_country'] = None
         if self.shipment_address and self.shipment_address.country:
             pattern['to_country'] = self.shipment_address.country.id
+        else:
+            pattern['to_country'] = None
         return pattern
 
     @fields.depends('warehouse', 'shipment_address')
@@ -206,7 +208,10 @@ class Sale:
         Currency = pool.get('currency.currency')
 
         cost, currency_id = 0, None
-        if self.carrier:
+        if (self.carrier
+                and any(l.quantity >= 0 for l in self.lines
+                    if l.type == 'line'
+                    and l.product and l.product.type != 'service')):
             with Transaction().set_context(self._get_carrier_context()):
                 cost, currency_id = self.carrier.get_sale_price()
 
@@ -310,21 +315,46 @@ class Sale:
         return invoice
 
 
-class SaleLine:
-    __metaclass__ = PoolMeta
+class SaleLine(metaclass=PoolMeta):
     __name__ = 'sale.line'
     shipment_cost = fields.Numeric('Shipment Cost', digits=price_digits)
 
     def _get_invoice_line_quantity(self):
         quantity = super(SaleLine, self)._get_invoice_line_quantity()
-        if (self.shipment_cost
-                and self.sale.shipment_cost_method == 'shipment'):
-            return 0
+        if self.shipment_cost:
+            if self.sale.shipment_cost_method == 'shipment':
+                return 0
+            elif (self.sale.shipment_cost_method == 'order'
+                    and self.sale.invoice_method == 'shipment'):
+                shipments = self.sale.shipments
+                if (not shipments
+                        or all(s.state != 'done' for s in shipments)):
+                    return 0
         return quantity
 
 
-class Promotion:
-    __metaclass__ = PoolMeta
+class ReturnSale(metaclass=PoolMeta):
+    __name__ = 'sale.return_sale'
+
+    def do_return_(self, action):
+        pool = Pool()
+        Sale = pool.get('sale.sale')
+        SaleLine = pool.get('sale.line')
+        action, data = super(ReturnSale, self).do_return_(action)
+
+        return_sales = Sale.browse(data['res_id'])
+        lines = []
+        for sale in return_sales:
+            for line in sale.lines:
+                # Do not consider return shipment cost as a shipment cost
+                if line.shipment_cost:
+                    line.shipment_cost = None
+                    lines.append(line)
+        SaleLine.save(lines)
+        return action, data
+
+
+class Promotion(metaclass=PoolMeta):
     __name__ = 'sale.promotion'
 
     def get_context_formula(self, sale_line):
